/* -*- c -*-
 * Time-stamp: <2005-04-23 20:16:36 rsmith>
 * 
 * onepad/enpad.c
 * Copyright (C) 2001--2005 R.F. Smith <rsmith@xs4all.nl>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "padfile.h"

#ifndef NULL
#define NULL (void*)0
#endif

#ifndef RETURN_IF_FAIL
#define RETURN_IF_FAIL(test) if (!(test)) return
#endif

#ifndef RETURN_VAL_IF_FAIL
#define RETURN_VAL_IF_FAIL(test,val) if (!(test)) return ((val))
#endif

#ifndef NDEBUG
/* __FUNCTION__ is a GCC feature. */
#undef debug
#define debug(TXT) fprintf(stderr,"%s %s(): %s\n",__FILE__,__FUNCTION__,TXT)
#else
#undef debug
#define debug(TXT) (void)0
#endif				/* NDEBUG */

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: enpad.c 43 2005-04-23 18:19:09Z rsmith $";
#endif

extern void print_license(FILE * f);

int main(int argc, char *argv[])
{
	struct padfile *key, *cypher;
	FILE *cleartext;
	unsigned char in, out, keyb;
	int rv = 0;

	/* check number of commend-line arguments */
	if (argc != 4) {
		printf("enpad " VERSION
		       " Copyright (C) 2001--2005 by R.F. Smith "\
		       "<rsmith@xs4all.nl>\n");
		printf("Usage: enpad keyfile cleartext cyphertext\n\n");
		print_license(stdout);
		return 0;
	}

	/* open the key file */
	key = pfopenr(argv[1]);
	if (key == NULL) {
		fprintf(stderr, "enpad: error opening key file\n");
		return 1;
	}
	
	/* open the cleartext file */
	cleartext = fopen(argv[2], "r");
	if (cleartext == NULL) {
		printf("enpad: unable to open cleartext file %s\n",
		       argv[2]);
		return 2;
	}

	/* open the cyphertext file */
	cypher = pfopenw(argv[3], key->id);
	if (cypher == NULL) {
		fprintf(stderr, "enpad: error opening cyphertext file\n");
		return 3;
	}

	/* encrypt the cleartext, write the cyphertext. */
	while (fread(&in, sizeof(unsigned char), 1, cleartext) == 1) {
		if (pfreadb(key, &keyb)) {
			rv = 4;
			fprintf(stderr, "enpad: message clipped\n");
			break;
		}
		out = in ^ keyb;
		if (pfwriteb(cypher, out)) {
			rv = 5;
			fprintf(stderr, "enpad: error writing cyphertext\n");
			break;
		}
	}

	/* close the files */
	pfclose(key);
	pfclose(cypher);
	fclose(cleartext);
	return rv;
}

/* EOF enpad.c */
